#!/usr/bin/python

# stereo VOX with literal rounding model

from io import open
from sys import argv
from struct import unpack, pack
from math import floor

infile = open(argv[1], "rb")
outfile = open(argv[2], "wb")

def write32(x): outfile.write(pack("<I",x))
def write16(x): outfile.write(pack("<H",x))
def write16signed(x): outfile.write(pack("<h",x))

assert(infile.read(4) == "ADP\x02")
data_len = unpack(">I",infile.read(4))[0]

infile.seek(0x10)

data = infile.read(data_len)
infile.close()

channels = 2
sample_rate = 44100

# write RIFF headers
outfile.write("RIFF")
write32(data_len*4+0x24)  # total size
outfile.write("WAVEfmt ")

write32(0x10)     # fmt chunk size
write16(0x1)      # ima codec id
write16(channels) # channels
write32(sample_rate)
write32(sample_rate*channels*2)
write16(2)  # block align
write16(16)  # bits per sample

outfile.write("data")
write32(data_len*4)

hist = [0,0]
idx = [0,0]

VOX_Adj = \
[
    -1, -1, -1, -1, 2, 4, 6, 8,
    -1, -1, -1, -1, 2, 4, 6, 8 
]

VOX_Step = \
[
    16, 17, 19, 21, 23, 25, 28, 31,
    34, 37, 41, 45, 50, 55, 60, 66,
    73, 80, 88, 97, 107, 118, 130, 143,
    157, 173, 190, 209, 230, 253, 279, 307,
    337, 371, 408, 449, 494, 544, 598, 658,
    724, 796, 876, 963, 1060, 1166, 1282, 1411,
    1552
]

def decode_vox(n, c):
    global idx, hist
    stepsize = VOX_Step[idx[c]]

    """
    delta = stepsize >> 3
    if (n&1): delta += stepsize >> 2
    if (n&2): delta += stepsize >> 1
    if (n&4): delta += stepsize
    if (n&8):
        newsample = max(-2048, hist[c] - delta)
    else:
        newsample = min(2047, hist[c] + delta)
    """

    delta = ((n&7) + 0.5) * stepsize / 4
    if (n&8):
        newsample = max(-2048, hist[c] - delta)
    else:
        newsample = min(2047, hist[c] + delta)

    idx[c] = max(0, min(48, idx[c] + VOX_Adj[n]))

    return newsample

for i in range(len(data)):
    b_in = ord(data[i])

    hist[0] = decode_vox((b_in & 0xF0)>>4, 0)
    write16signed(hist[0]*16)

    hist[1] = decode_vox(b_in & 0xF, 1)
    write16signed(hist[1]*16)

outfile.close()

